/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.apache.cxf.common.util;

/**
 * Utility class for checking well-known system properties
 *
 */
public final class SystemUtils {
    
    public static final String SPRING_VALIDATION_MODE = "org.apache.cxf.spring.validation.mode";
    
    private SystemUtils() {
        
    }

    /**
     * Gets org.apache.cxf.spring.validation.mode property value if available 
     * @return Spring validation mode
     */
    public static String getSpringValidationMode() {
        String mode = System.getProperty(SPRING_VALIDATION_MODE);
        if (mode == null) {
            mode = System.getProperty("spring.validation.mode");
        }
        return mode;
    }
    
}
